@@ -12,17 +12,27 @@ package Jedi;
 
 use Moo;
 
-our $VERSION = '0.04';    # VERSION
+our $VERSION = '0.05';    # VERSION
 
 use Jedi::Helpers::Scalar;
 use Jedi::Request;
 use Jedi::Response;
+use CHI;
 
 use Module::Runtime qw/use_module/;
 use Carp qw/croak/;
 
 has '_jedi_roads'           => ( is => 'ro', default => sub { [] } );
 has '_jedi_roads_is_sorted' => ( is => 'rw', default => sub {0} );
+has '_jedi_roads_cache' => ( is => 'lazy', clearer => 1 );
+
+sub _build__jedi_roads_cache {
+    return CHI->new(
+        driver    => 'RawMemory',
+        datastore => {},
+        max_size  => 10_000
+    );
+}
 
 sub road {
     my ( $self, $base_route, $module ) = @_;
@@ -35,6 +45,7 @@ sub road {
 
     push( @{ $self->_jedi_roads }, [ $base_route => $jedi ] );
     $self->_jedi_roads_is_sorted(0);
+    $self->_clear_jedi_roads_cache;
     return;
 }
 
@@ -52,9 +63,22 @@ sub response {
     my $path_info = $env->{PATH_INFO}->full_path();
     my $response  = Jedi::Response->new();
 
+    if ( my $road_def = $self->_jedi_roads_cache->get($path_info) ) {
+        my ( $road, $jedi ) = @$road_def;
+        return $jedi->response(
+            Jedi::Request->new(
+                jedi_env => $jedi_env,
+                env      => $env,
+                path     => $path_info->without_base($road)
+            ),
+            $response
+        );
+    }
+
     for my $road_def (@$sorted_roads) {
         my ( $road, $jedi ) = @$road_def;
         if ( $path_info->start_with($road) ) {
+            $self->_jedi_roads_cache->set( $path_info => $road_def );
             return $jedi->response(
                 Jedi::Request->new(
                     jedi_env => $jedi_env,
@@ -86,7 +110,7 @@ Jedi - Jedi Web Framework
 
 =head1 VERSION
 
-version 0.04
+version 0.05
 
 =head1 DESCRIPTION
 
