@@ -11,15 +11,22 @@ package Jedi::Role::App;
 # ABSTRACT: Jedi App Role
 
 use Moo::Role;
-
-our $VERSION = '0.04';    # VERSION
-
+our $VERSION = '0.05';    # VERSION
 use Jedi::Helpers::Scalar;
-
+use CHI;
 use Carp qw/carp croak/;
 
 has '_jedi_routes'  => ( is => 'ro', default => sub { {} } );
 has '_jedi_missing' => ( is => 'ro', default => sub { [] } );
+has '_jedi_routes_cache' => ( is => 'lazy', clearer => 1 );
+
+sub _build__jedi_routes_cache {
+    return CHI->new(
+        driver    => 'RawMemory',
+        datastore => {},
+        max_size  => 10_000
+    );
+}
 
 sub _jedi_routes_push {
     my ( $self, $which, $path, $sub ) = @_;
@@ -29,6 +36,7 @@ sub _jedi_routes_push {
     croak "sub invalid !"  if ref $sub ne 'CODE';
     $path = $path->full_path if ref $path ne 'Regexp';
     push @{ $self->_jedi_routes->{$which} }, [ $path, $sub ];
+    $self->_clear_jedi_routes_cache;
     return;
 }
 
@@ -57,29 +65,38 @@ sub missing {
     croak "sub invalid !" if ref $sub ne 'CODE';
 
     push( @{ $self->_jedi_missing }, $sub );
+    $self->_clear_jedi_routes_cache;
     return;
 }
 
 sub response {
     my ( $self, $request, $response ) = @_;
 
-    my $path    = $request->path;
-    my $routes  = $self->_jedi_routes->{ $request->env->{REQUEST_METHOD} };
-    my $methods = [];
+    my $path   = $request->path;
+    my $routes = $self->_jedi_routes->{ $request->env->{REQUEST_METHOD} };
+    my $methods;
 
-    if ( ref $routes eq 'ARRAY' ) {
-        for my $route_def (@$routes) {
-            my ( $route, $sub ) = @$route_def;
-            if ( ref $route eq 'Regexp' ) {
-                push @$methods, $sub if $path =~ $route;
-            }
-            else {
-                push @$methods, $sub if $path eq $route->full_path;
+    if ( my $cache_routes = $self->_jedi_routes_cache->get($path) ) {
+        $methods = $cache_routes;
+    }
+    else {
+        $methods = [];
+        if ( ref $routes eq 'ARRAY' ) {
+            for my $route_def (@$routes) {
+                my ( $route, $sub ) = @$route_def;
+                if ( ref $route eq 'Regexp' ) {
+                    push @$methods, $sub if $path =~ $route;
+                }
+                else {
+                    push @$methods, $sub if $path eq $route->full_path;
+                }
             }
         }
-    }
 
-    @$methods = @{ $self->_jedi_missing } if !scalar @$methods;
+        @$methods = @{ $self->_jedi_missing } if !scalar @$methods;
+
+        $self->_jedi_routes_cache->set( $path => $methods );
+    }
 
     for my $meth (@$methods) {
         last if !$self->$meth( $request, $response );
@@ -100,7 +117,7 @@ Jedi::Role::App - Jedi App Role
 
 =head1 VERSION
 
-version 0.04
+version 0.05
 
 =head1 DESCRIPTION
 
